const {Command, flags} = require('@oclif/command')
var config = require('../lib/config');
var client = require('../lib/client');
const ctable = require('console.table');

class ReportsCommand extends Command {
    async run() {
        var api_key = null;
        const {flags} = this.parse(ReportsCommand)
        if (flags.app == null) {
            console.log("app is null");
            return;
        }
        if (flags.suite == null) {
            console.log("Set a test suite to run");
            return;
        }
        if (flags.device == null) {
            console.log("Set a device group");
            return;
        }
        if (flags.name == null) {
            console.log("Set name for this test run");
            return;
        }
        config().then(function(config) {
            api_key = config['api_key'];
            return client.get('apps/'+flags.app, {
                headers: {'X-API-KEY': api_key}
            });
        }).then(function(response) {
            var versions = response.data.versions;
            var labels = response.data.labels;
            console.log("Selecting " + labels[labels.length-1]);
            var version = versions[versions.length-1]
            var params = {
                report_name: flags.name,
                suite_id: parseInt(flags.suite),
                device_group_id: parseInt(flags.device),
            }
            return client.post('apps/'+flags.app +'/version/'+version+'/test/report', params, {
                headers: {'X-API-KEY': api_key}
            });
        }).then(function (response) {
            var table_data = [];
            table_data.push({
                'name': response.data.report_name,
                'status': response.data.status,
            });
                    
            const table = ctable.getTable(table_data);
            console.log(table);
        }).catch(function (err) {
            console.log(err.response.data.message || err.message);
        }, function (err) {
            console.log(err.message);
        });
      }
  }
  
  ReportsCommand.description = `Schedule tests on the latest uploaded version`
  
  ReportsCommand.flags = {
    app: flags.string({char: 'a', description: 'app id'}),
    suite: flags.string({char: 's', description: 'test suite id'}),
    device: flags.string({char: 'd', description: 'device group id'}),
    name: flags.string({char: 'n', description: 'report name'}),
  }
  
  module.exports = ReportsCommand
  